/*
  include file for macro package.  functions return
  pointer to message for error, null pointer for success.
*/

#if (!defined(H_MACRO))
#define H_MACRO

#if defined(MCR_FILE)
/* in implementation file for package */

/* final result for caller */
char *mcr_result;
/* free spaces in final result area */
int mcr_n_result;

#else

/* define for caller */
extern char *mcr_result;
extern int mcr_n_result;

#endif


/*
  initialization function.  init_str_alloc() must
  be called before this function.
*/
char *mcr_init(void);


/*
  define a macro
*/
char *mcr_def
  (
    /* name of macro */
    char *name,
    /* body of definition (cannot be altered after call) */
    void *mval,
    /* if magic != 0, the body of the macro is a string.  if magic
       = 0, body is a pointer for a function whose proto-type is
       char *f(int n_arg,char **arg);
       this function is called when the macro is invoked, with
       the number of arguments to the macro and the
       array of arguments.  The first argument is the name
       of the macro.  f must return null for success, an error
       message string for failure. */
    int magic
  );


/*
  dump names in macro table
*/
void mcr_dump(void);


/*
  function to begin expansion
*/
void mcr_start_expand
  (
    /* number of top level arguments */
    int n_orig_arg,
    /* array of top level arguments */
    char **orig_arg
  );


/*
  insert a character directly into the output
  stream without evaluating it
*/
char *mcr_noeval_char
  (
    char c
  );


/*
  next character to evaluate.
*/
char *mcr_next_char
  (
    char c
  );


/*
  boolean function - returns non-zero if in midst of
  a macro expansion.
*/
int mcr_expanding(void);
