
/*
  functions to allocate strings efficiently
*/

#include "stralloc.h"

/* pointer to storage from which strings can be allocated */
static char *heap;
/* number of bytes available in heap */
static int n_heap;

/* number of bytes to put into heap each time it is emptied */
#define N_INIT_HEAP (16*1024)


/*
  initialize for string allocation
*/
void init_str_alloc
  (
    void
  )
  {
    n_heap = 0;

    return;
  }

/*
  allocate a string.  returns null if cannot allocate string
  or n is zero.
*/
char *str_alloc
  (
    /* number of characters to allocate */
    int n
  )
  {
    if (n == 0)
      return((char *) 0);

    if (n > n_heap)
      /* refill heap */
      {
        if (n > N_INIT_HEAP)
          return((char *) 0);

        heap = (char *) malloc((size_t) N_INIT_HEAP);
        if (heap == (char *) 0)
          /* malloc error */
          {
            n_heap = 0;
            return((char *) 0);
          }

        n_heap = N_INIT_HEAP;
        /* move heap pointer to end of storage and allocate
           "backwards" */
        heap += N_INIT_HEAP;
      }

    n_heap -= n;
    heap -= n;
    return(heap);
  }
